// Constants
const int kNumLights = 4;

const vec3 ambientColor = vec3(1.0, 1.0, 1.0);
const vec3 specColor = vec3(1.0, 1.0, 1.0);

uniform float ambientFactor;
uniform mat4 viewMatrix;

// Light colors and positions
uniform vec3 lightColors[kNumLights];
uniform vec3 lightPositions[kNumLights];
uniform float lightRadii[kNumLights];

uniform vec4 objectColor;

in vec3 position;
in vec2 texCoord;
in vec3 normal;

out vec4 fragColor;

void main (void)
{
    // -- Begin Phong lighting model -------------------------------------------
    vec3 ambient = objectColor.rgb * ambientColor * ambientFactor;
    vec3 diffuse = vec3(0.0, 0.0, 0.0);
    vec3 specular = vec3(0.0, 0.0, 0.0);

    for (int i = 0; i < kNumLights; i++)
    {
        vec3 viewLightPosition = (viewMatrix * vec4(lightPositions[i], 1.0)).xyz;

        vec3 incident = normalize(viewLightPosition - position);
        float lambert = max(0.0, dot(incident, normal));

        float dist = length(viewLightPosition - position);
        float atten = 1.0 - clamp(dist / lightRadii[i], 0.0, 1.0);
        vec3 viewDir = normalize(-position);
        vec3 halfDir = normalize(incident + viewDir);

        float rFactor = max(0.0, dot(halfDir, normal));
        float sFactor = pow(rFactor, 32.0);

        diffuse += objectColor.rgb * lightColors[i] * lambert * atten;

        vec3 specFromThisLight = lightColors[i] * sFactor * atten;

        specular += specFromThisLight;
    }
    // -- End Phong lighting model --------------------------------------------

    fragColor = vec4(ambient + diffuse + specular, objectColor.a);
}