uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projMatrix;
uniform mat4 texMatrix;

layout(location = 0) in vec3 inPosition;
layout(location = 1) in vec4 inColor;
layout(location = 2) in vec2 inTexCoord;
layout(location = 3) in vec3 inNormal;
layout(location = 4) in vec3 inTangent;
layout(location = 5) in vec3 inBitangent;

out vec3 position;
out vec3 worldPos;
out vec3 viewPos;
out vec4 color;
out vec2 texCoord;
out vec3 normal;
out vec3 tangent;
out vec3 bitangent;

void main(void)
{
    mat3 normalMatrix = transpose(inverse(mat3(viewMatrix * modelMatrix)));

    position = inPosition;
    worldPos = (modelMatrix * vec4(inPosition, 1.0)).xyz;
    viewPos = (viewMatrix * vec4(worldPos, 1.0)).xyz;

    color = inColor;
    texCoord = (texMatrix * vec4(inTexCoord, 0.0, 1.0)).xy;
    
    normal = normalize(normalMatrix * normalize(inNormal));
    tangent = normalize(normalMatrix * normalize(inTangent));
    bitangent = normalize(normalMatrix * normalize(inBitangent));

    gl_Position	= projMatrix * vec4(viewPos, 1.0);
}