uniform mat4 texMatrix;

uniform sampler2D texDiffuse;
uniform sampler2D texNormal;
uniform sampler2D texSpecular;

uniform sampler2D texSpecial1;
uniform sampler2D texSpecial2;

in vec3 position;
in vec3 worldPos;
in vec3 viewPos;
in vec4 color;
in vec2 texCoord;
in vec3 normal;
in vec3 tangent;
in vec3 bitangent;

layout(location = 0) out vec4 outGPosition;
layout(location = 1) out vec4 outGNormal;
layout(location = 2) out vec4 outGAlbedo;
layout(location = 3) out vec4 outGMaterial;

uniform bool lightingEnabled;

void main (void)
{
    // Scale texture coordinate for diffuse/normal
    vec2 scaledTexCoord = (texMatrix * vec4(texCoord, 0.0, 1.0)).xy;

    // Use texSpecial2 as a map to fade between texDiffuse and texSpecial1 to add grassy areas to the desert
    float whichDiffuse = texture(texSpecial2, texCoord).r;
    vec4 diffuseColor = mix(texture(texDiffuse, scaledTexCoord), texture(texSpecial1, scaledTexCoord), whichDiffuse);

    // Darken underwater areas of heightmap
    float underWaterDarkenHeight = 2.2;
    if (worldPos.y < underWaterDarkenHeight)
        diffuseColor.rgb *= mix(0.5, 1.0, worldPos.y / underWaterDarkenHeight);
       
    // Normal mapping
    mat3 tbn = mat3(tangent, bitangent, normal);
    vec3 mappedNormal = normalize(tbn * normalize(texture(texNormal, scaledTexCoord).rgb * 2.0 - 1.0));

    // Viewspace position to position texture
    outGPosition = vec4(viewPos, 1.0);

    // Viewspace normal and specular power to normal/specular texture
    outGNormal = vec4(normalize(mappedNormal), 1.0);

    // Albedo color to albedo texture
    outGAlbedo = diffuseColor;

    // Lighting enable flag to material buffer
    outGMaterial.r = lightingEnabled ? 1.0 : 0.0;

    // Specular power to material buffer
    outGMaterial.g = texture(texSpecular, texCoord).r;
    outGMaterial.a = 1.0;
}