uniform sampler2D texDiffuse;
uniform samplerCube texEnvMap;

uniform vec3 cameraPosition;

in vec3 worldPos;
in vec3 viewPos;
in vec3 worldNormal;
in vec3 viewNormal;

layout(location = 0) out vec4 outGPosition;
layout(location = 1) out vec4 outGNormal;
layout(location = 2) out vec4 outGAlbedo;
layout(location = 3) out vec4 outGMaterial;

uniform bool lightingEnabled;

void main (void)
{
    // Color
   //vec3 viewVector = normalize(worldPos - cameraPosition);
    vec3 viewVector = normalize(cameraPosition - worldPos);
    vec3 reflectedVector = reflect(viewVector, worldNormal);
    vec3 refractedVector = refract(viewVector, worldNormal, 1.0/1.33);

    // HACK: Again, no idea what's up with my cube mapping maths
    // I think my Camera class is junk.
    reflectedVector.x *= -1.0;
    refractedVector.x *= -1.0;

    //vec4 diffuseColor = vec4(mix(vec3(0.0, 1.0, 1.0), texture(texEnvMap, reflectedVector).rgb, 0.8), 1.0);
    vec4 reflectedColor = texture(texEnvMap, reflectedVector);
    vec4 refractedColor = texture(texEnvMap, refractedVector);
    vec4 diffuseColor = mix(reflectedColor, refractedColor, 0.2);
    diffuseColor = mix(diffuseColor, vec4(0.0, 0.7, 1.0, 0.9), 0.2);
    diffuseColor.a = 1;

    // Viewspace position to position texture
    outGPosition = vec4(viewPos, 1.0);

    // Viewspace normal and specular power to normal/specular texture
    outGNormal = vec4(viewNormal, 1.0);

    // Albedo color to albedo texture
    outGAlbedo = diffuseColor;

    // Lighting enable flag to material buffer
    outGMaterial.r = lightingEnabled ? 1.0 : 0.0;

    // Specular power and enable lighting
    outGMaterial.r = 1.0;
    outGMaterial.g = 1.0;
    outGMaterial.a = 1.0;
}