layout (triangles, equal_spacing, ccw) in;

uniform sampler2D texSpecial1; // used as heightmap

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projMatrix;
uniform mat4 texMatrix;

in vec3 tcsWorldPos[];
in vec3 tcsViewPos[];
in vec2 tcsTexCoord[];
in vec3 tcsNormal[];
in vec3 tcsTangent[];
in vec3 tcsBitangent[];

out vec3 tesWorldPos;
out vec3 tesViewPos;
out vec2 tesTexCoord;
out vec3 tesNormal;
out vec3 tesTangent;
out vec3 tesBitangent;

vec3 interpolate3(vec3 a, vec3 b, vec3 c) {
	return gl_TessCoord.x * a + gl_TessCoord.y * b + gl_TessCoord.z * c;
}

vec2 interpolate2(vec2 a, vec2 b, vec2 c) {
	return gl_TessCoord.x * a + gl_TessCoord.y * b + gl_TessCoord.z * c;
}

void main()
{
	// Interpolate output values
	vec3 combinedPos = interpolate3(gl_in[0].gl_Position.xyz,
									gl_in[1].gl_Position.xyz,
									gl_in[2].gl_Position.xyz);

	//tesWorldPos = interpolate3(tcsWorldPos[0], tcsWorldPos[1], tcsWorldPos[2]); // redundant?
	tesViewPos = interpolate3(tcsViewPos[0], tcsViewPos[1], tcsViewPos[2]);
	tesTexCoord = interpolate2(tcsTexCoord[0], tcsTexCoord[1], tcsTexCoord[2]);
	tesNormal = -interpolate3(tcsNormal[0], tcsNormal[1], tcsNormal[2]); // redundant?
	tesTangent = interpolate3(tcsTangent[0], tcsTangent[1], tcsTangent[2]);
	tesBitangent = interpolate3(tcsBitangent[0], tcsBitangent[1], tcsBitangent[2]);

	vec4 position = vec4(combinedPos, 1.0);

	// Calculate a normal for this subdivided triangle
	vec3 a = gl_in[0].gl_Position.xyz;
	vec3 b = gl_in[1].gl_Position.xyz;
	vec3 c = gl_in[2].gl_Position.xyz;
	vec3 normal = normalize(cross(c - b, a - b));

	// Negate, as we're *INSIDE* the cube
	normal *= -1.0;

	// Get the height from the heightmap
	float height = texture(texSpecial1, tesTexCoord).r;

	// Only apply the heightmap if it is significant enough
	// (helps keep "cement" at 0 so that edges of cube stay stitched together)
	if (height > 0.5)
	{
		height /= 50.0;
		position.xyz += (normal * vec3(height, height, height));
		tesViewPos.xyz += (tesNormal * vec3(height, height, height));
	}
	tesWorldPos = (modelMatrix * position).xyz;

	// TODO: Output normals aren't adjusted, texture coordinates don't work. Aaaargh!
	// My brain has been tessellated to hell and back and deadline is approaching.
	// Hacky will have to do. :(

	gl_Position = projMatrix * viewMatrix * modelMatrix * position;
}