uniform mat4 modelMatrix;
uniform mat4 viewMatrix;

layout(location = 0) in vec3 inPosition;
layout(location = 1) in vec4 inColor;
layout(location = 2) in vec2 inTexCoord;
layout(location = 3) in vec3 inNormal;
layout(location = 4) in vec3 inTangent;
layout(location = 5) in vec3 inBitangent;

out vec3 vsWorldPos;
out vec3 vsViewPos;
out vec2 vsTexCoord;
out vec3 vsNormal;
out vec3 vsTangent;
out vec3 vsBitangent;

void main(void)
{
    mat3 normalMatrix = transpose(inverse(mat3(viewMatrix * modelMatrix)));

    vsWorldPos = (modelMatrix * vec4(inPosition, 1.0)).xyz;
    vsViewPos = (viewMatrix * vec4(vsWorldPos, 1.0)).xyz;

    vsTexCoord = inTexCoord;
    
    vsNormal = normalize(normalMatrix * normalize(inNormal));
    vsTangent = normalize(normalMatrix * normalize(inTangent));
    vsBitangent = normalize(normalMatrix * normalize(inBitangent));

    // No projection
    gl_Position	= vec4(inPosition.xyz, 1.0);
}