// ----------------------------------------------------------------------------
// GaussianBlur.vert
// Dale Whinham / 130343550
// 
// Implements a separable Gaussian blur.
// 
// Use 'vertical' bool uniform to do separate horizontal and vertical passes.
// ----------------------------------------------------------------------------

// Needs to be odd so that there is a centre texel, e.g. 11 = 5 samples either side of centre
const int kGaussianBlurKernelSize = 11;

uniform sampler2D texInput;
uniform bool vertical;

// Actually a combination of 2D position and texture coordinates
layout(location = 0) in vec4 inPosition;

out vec2 texCoords[kGaussianBlurKernelSize];

void main(void)
{
    // Get base texture coordinate from .zw component
    vec2 texCoord = inPosition.zw;

    // Calculate texture coordinates
    int halfKernelSize = int(kGaussianBlurKernelSize / 2.0);

    // Texel size is 1/width or 1/height depending on whether we're doing a horizontal or vertical pass
    vec2 texDimensions = textureSize(texInput, 0);
    float texelSize = 1.0 / (vertical ? texDimensions.y : texDimensions.x);

    if (vertical)
        for (int i = -halfKernelSize; i <= halfKernelSize; ++i)
            texCoords[i + halfKernelSize] = texCoord + vec2(0.0, texelSize * i);
    else
        for (int i = -halfKernelSize; i <= halfKernelSize; ++i)
            texCoords[i + halfKernelSize] = texCoord + vec2(texelSize * i, 0.0);

    gl_Position = vec4(inPosition.xy, 0.0, 1.0);
}
