// ----------------------------------------------------------------------------
// GaussianBlurSingleChannel.frag
// 
// Identical to GaussianBlur.frag, except we sample a single channel of the
// input texture, and output in monochrome.
// ----------------------------------------------------------------------------

// Needs to be odd so that there is a centre texel, e.g. 11 = 5 samples either side of centre
const int kGaussianBlurKernelSize = 11;

// Generated using http://dev.theomader.com/gaussian-kernel-calculator/
const float kGaussianBlurKernel[kGaussianBlurKernelSize] = float[]
(
    0.0093,
    0.028002,
    0.065984,
    0.121703,
    0.175713,
    0.198596,
    0.175713,
    0.121703,
    0.065984,
    0.028002,
    0.0093
);

uniform sampler2D texInput;

in vec2 texCoords[kGaussianBlurKernelSize];

out float fragColor;

void main(void)
{
    fragColor = 0.0;
    for (int i = 0; i < kGaussianBlurKernelSize; ++i)
    {
        fragColor += texture(texInput, texCoords[i]).r * kGaussianBlurKernel[i];
    }
}