uniform sampler2D texGPosition;
uniform sampler2D texGNormal;
uniform sampler2D texSSAONoise;

uniform mat4 viewMatrix;
uniform mat4 projMatrix;

uniform vec3 ssaoSamples[128];
uniform int ssaoKernelSize;
uniform float ssaoRadius;

in vec2 texCoord;

out float fragColor;

const float bias = 0.0005;
const int kNoiseTexSize = 4;

void main (void)
{
    // Noise texture tiling; based on G-buffer (screen) dimensions and
    // dimensions of noise texture itself
    vec2 noiseTexScale = vec2(textureSize(texGPosition, 0)) / float(kNoiseTexSize);

    vec3 position = texture(texGPosition, texCoord).xyz;
    vec3 normal = texture(texGNormal, texCoord).rgb;
    vec3 randomVec = texture(texSSAONoise, texCoord * noiseTexScale).xyz;

    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = cross(normal, tangent);
    mat3 tbn = mat3(tangent, bitangent, normal);

    float occlusion = 0.0;
    for (int i = 0; i < ssaoKernelSize; i++)
    {
        // Use Gram-Schmidt process to move sample from tangent space to view space
        vec3 samp = tbn * ssaoSamples[i]; 
        samp = position + samp * ssaoRadius; 
        
        // Now project into clip space
        vec4 offset = projMatrix * vec4(samp, 1.0);

        // Apply the perspective divide
        offset.xyz /= offset.w;

        // Transform to range 0.0 - 1.0 (G-buffer texture coordinate range)
        offset.xyz  = offset.xyz * 0.5 + 0.5;

        float sampleDepth = texture(texGPosition, offset.xy).z; 
        
        // range check & accumulate
        float rangeCheck = smoothstep(0.0, 1.0, ssaoRadius / abs(position.z - sampleDepth));
        //float rangeCheck = abs(position.z - sampleDepth) < ssaoRadius ? 1.0 : 0.0;
        occlusion += (sampleDepth >= samp.z + bias ? 1.0 : 0.0) * rangeCheck;   
    }

    occlusion = 1.0 - (occlusion / float(ssaoKernelSize));

    fragColor = occlusion;
}
